@@ -18,7 +18,6 @@ Each property type (association, attribute and enumeration) has three specific
 methods:
     _get():           return the value
     _set(value):      set the value or add it to a list
-    (_set2(value):    used by association, used to set bi-directional ass.)
     _del(value=None): delete the value. 'value' is used to tell which value
                       is to be removed (in case of associations with
                       multiplicity > 1).
@@ -31,9 +30,23 @@ __all__ = [ 'attribute', 'enumeration', 'association', 'derivedunion', 'redefine
 
 from collection import collection
 import operator
+from gaphor.undomanager import get_undo_manager
 
 #infinite = 100000
 
+class undoaction(object):
+    """This undo action contains one undo action for an attribute
+    property.
+    """
+
+    def __init__(self, prop, obj, value):
+        self.prop = prop
+        self.obj = obj
+        self.value = value
+        #print 'adding new property', prop.name, obj, value
+        get_undo_manager().add_undo_action(self)
+
+
 class umlproperty(object):
     """Superclass for attribute, enumeration and association.
     The subclasses should define a 'name' attribute that contains the name
@@ -100,6 +113,21 @@ class umlproperty(object):
                         log.error(e, e)
 
 
+class undoattributeaction(undoaction):
+    """This undo action contains one undo action for an attribute
+    property.
+    """
+
+    def undo(self):
+        self.redo_value = self.prop._get(self.obj)
+        setattr(self.obj, self.prop._name, self.value)
+        self.prop.notify(self.obj)
+
+    def redo(self):
+        setattr(self.obj, self.prop._name, self.redo_value)
+        self.prop.notify(self.obj)
+
+
 class attribute(umlproperty):
     """Attribute.
     Element.attr = attribute('attr', types.StringType, '')"""
@@ -118,10 +146,6 @@ class attribute(umlproperty):
         #print 'attribute.load:', self.name, self.type, value,
         if self.type is not object:
             value = self.type(value)
-        #if not isinstance(value, self.type):
-        #    if type(self.type) is not type(()):
-        #    else:
-        #        raise AttributeError, 'Value should be of type %s (is %s)' % (self.type.__name__, type(value))
         setattr(obj, self._name, value)
 
     def __str__(self):
@@ -139,7 +163,12 @@ class attribute(umlproperty):
     def _set(self, obj, value):
         if value is not None and not isinstance(value, self.type):
             raise AttributeError, 'Value should be of type %s' % hasattr(self.type, '__name__') and self.type.__name__ or self.type
-        #self.old = self._get(obj)
+
+        if value == self._get(obj):
+            return
+
+        undoattributeaction(self, obj, self._get(obj))
+
         if value == self.default and hasattr(obj, self._name):
             delattr(obj, self._name)
         else:
@@ -149,6 +178,7 @@ class attribute(umlproperty):
     def _del(self, obj, value=None):
         #self.old = self._get(obj)
         try:
+            undoattributeaction(self, obj, self._get(obj))
             delattr(obj, self._name)
         except AttributeError:
             pass
@@ -184,6 +214,7 @@ class enumeration(umlproperty):
         if not value in self.values:
             raise AttributeError, 'Value should be one of %s' % str(self.values)
         if value != self._get(obj):
+            undoattributeaction(self, obj, self._get(obj))
             if value == self.default:
                 delattr(obj, self._name)
             else:
@@ -192,6 +223,7 @@ class enumeration(umlproperty):
 
     def _del(self, obj, value=None):
         try:
+            undoattributeaction(self, obj, self._get(obj))
             delattr(obj, self._name)
         except AttributeError:
             pass
@@ -199,6 +231,32 @@ class enumeration(umlproperty):
             self.notify(obj)
 
 
+class undosetassociationaction(undoaction):
+    """Undo a 'set' action in an association.
+    """
+
+    def undo(self):
+        #log.debug('undosetassociationaction del: %s %s %s' % (self.obj, self.prop.name, self.value))
+        self.prop._del(self.obj, self.value)
+
+    def redo(self):
+        #log.debug('undosetassociationaction set: %s %s %s' % (self.obj, self.prop.name, self.value))
+        self.prop._set(self.obj, self.value)
+
+
+class undodelassociationaction(undoaction):
+    """Undo a 'del' action in an association.
+    """
+
+    def undo(self):
+        #log.debug('undodelassociationaction set: %s %s %s' % (self.obj, self.prop.name, self.value))
+        self.prop._set(self.obj, self.value)
+
+    def redo(self):
+        #log.debug('undodelassociationaction del: %s %s %s' % (self.obj, self.prop.name, self.value))
+        self.prop._del(self.obj, self.value)
+
+
 class association(umlproperty):
     """Association, both uni- and bi-directional.
     Element.assoc = association('assoc', Element, opposite='other')
@@ -223,16 +281,8 @@ class association(umlproperty):
     def load(self, obj, value):
         if not isinstance(value, self.type):
             raise AttributeError, 'Value for %s should be of type %s (%s)' % (self.name, self.type.__name__, type(value).__name__)
-        if self._set2(obj, value) and self.opposite:
-            opposite = getattr(type(value), self.opposite)
-            if opposite.upper > 1:
-                if not obj in opposite._get(value):
-                    #print 'Setting opposite*:', self.name, str(opposite), obj, value
-                    opposite._set2(value, obj)
-            else:
-                if not obj is opposite._get(value):
-                    #print 'Setting opposite1:', self.name, obj, value
-                    opposite._set2(value, obj)
+        # TODO: avoid sending notifications:
+        self._set(obj, value, do_notify=False)
 
     def postload(self, obj):
         """In the postload step, ensure that bi-directional associations
@@ -256,58 +306,6 @@ class association(umlproperty):
             s += ' %s-> %s' % (self.composite and '<>' or '', self.opposite)
         return s + '>'
 
-    def __get__(self, obj, class_=None):
-        """Retrieve the value of the association. In case this is called
-        directly on the class, return self."""
-        #print '__get__', self, obj, class_
-        if not obj:
-            return self
-        return self._get(obj)
-
-    def __set__(self, obj, value):
-        """Set a new value for our attribute. If this is a collection, append
-        to the existing collection."""
-        #print '__set__', self, obj, value
-        if not (isinstance(value, self.type) or \
-                (value is None and self.upper == 1)):
-            raise AttributeError, 'Value should be of type %s' % self.type.__name__
-        # Remove old value only for uni-directional associations
-        if self.upper == 1:
-            old = self._get(obj)
-            # do nothing if we are assigned our current value:
-            if value is old:
-                return
-            if old:
-                self.__delete__(obj, old)
-            if value is None:
-                #self.notify(obj)
-                return
-
-        # if we needed to set our own side, set the opposite
-        # Call _set2() so we can make sure the opposite side is set before
-        # a signal is emited.
-        if self._set2(obj, value):
-            # Set opposite side.
-            # Use type(value) since the property may be overridden:
-            if self.opposite:
-                getattr(type(value), self.opposite)._set(value, obj)
-            self.notify(obj)
-
-    def __delete__(self, obj, value=None):
-        """Delete is used for element deletion and for removal of
-        elements from a list.
-        """
-        #print '__delete__', self, obj, value
-        if self.upper > 1 and not value:
-            raise Exception, 'Can not delete collections'
-        if not value:
-            value = self._get(obj)
-            if value is None:
-                return
-        if self.opposite:
-            getattr(type(value), self.opposite)._del(value, obj)
-        self._del(obj, value)
-        
     def _get(self, obj):
         #print '_get', self, obj
         # TODO: Handle lower and add items if lower > 0
@@ -319,45 +317,87 @@ class association(umlproperty):
                 # add 
                 #c = collection(self, obj, self.type)
                 #setattr(obj, self._name, c)
-                return []
+                c = collection(self, obj, self.type)
+                setattr(obj, self._name, c)
+                return c
             else:
                 return None
 
-    def _set(self, obj, value):
-        #print '_set', self, obj, value
-        if not isinstance(value, self.type):
-            raise AttributeError, 'Value should be of type %s' % self.type.__name__
-        if self._set2(obj, value):
-            self.notify(obj)
+    def _set(self, obj, value, from_opposite=False, do_notify=True):
+        """Set a new value for our attribute. If this is a collection, append
+        to the existing collection.
 
-    def _set2(self, obj, value):
-        """Real setter, avoid doing the assertion check twice.
-        Return True if notification should be send, False otherwise."""
-        if self.upper > 1:
+        This method is called from the opposite association property.
+        """
+        #print '__set__', self, obj, value
+        if not (isinstance(value, self.type) or \
+                (value is None and self.upper == 1)):
+            raise AttributeError, 'Value should be of type %s' % self.type.__name__
+        # Remove old value only for uni-directional associations
+        if self.upper == 1:
+            old = self._get(obj)
+            # do nothing if we are assigned our current value:
+            if value is old:
+                return
+
+            # is done is _del(): undoassociationaction(self, obj, old)
+            if old:
+                self._del(obj, old)
+            if value is None:
+                return
+            if value is self._get(obj):
+                #log.debug('association: value already in obj: %s' % value)
+                return
+
+            if not from_opposite:
+                undosetassociationaction(self, obj, value)
+            setattr(obj, self._name, value)
+        else:
+            # Set the actual value
             c = self._get(obj)
             if not c:
                 c = collection(self, obj, self.type)
                 setattr(obj, self._name, c)
             elif value in c:
                 #log.debug('association: value already in obj: %s' % value)
-                return False
+                return
+
+            if not from_opposite:
+                undosetassociationaction(self, obj, value)
             c.items.append(value)
-        else:
-            if value is self._get(obj):
-                #log.debug('association: value already in obj: %s' % value)
-                return False
-            setattr(obj, self._name, value)
 
         # Callbacks are only connected if a new relationship has
         # been established.
         value.connect('__unlink__', self.__on_unlink, obj)
         if self.composite:
             obj.connect('__unlink__', self.__on_composite_unlink, value)
-        return True
+        
+        if not from_opposite and self.opposite:
+            getattr(type(value), self.opposite)._set(value, obj, from_opposite=True, do_notify=do_notify)
+
+        if do_notify:
+            self.notify(obj)
+
+    def _del(self, obj, value, from_opposite=False):
+        # TODO: move code to _del
+        """Delete is used for element deletion and for removal of
+        elements from a list.
+        """
+        #print '__delete__', self, obj, value
+
+        if not value:
+            if self.upper > 1:
+                raise Exception, 'Can not delete collections'
+            value = self._get(obj)
+            if value is None:
+                return
+
+        if not from_opposite:
+            undodelassociationaction(self, obj, value)
+
+        if not from_opposite and self.opposite:
+            getattr(type(value), self.opposite)._del(value, obj, from_opposite=True)
 
-    def _del(self, obj, value):
-        """Delete value from the association."""
-        #print '_del', self, obj, value
         if self.upper > 1:
             c = self._get(obj)
             if c:
@@ -374,6 +414,7 @@ class association(umlproperty):
             except:
                 pass
                 #print 'association._del: delattr failed for %s' % self.name
+
         value.disconnect(self.__on_unlink, obj)
         if self.composite:
             obj.disconnect(self.__on_composite_unlink, value)
@@ -397,9 +438,6 @@ class association(umlproperty):
             self.__delete__(obj, value)
             # re-establish unlink handler:
             value.connect('__unlink__', self.__on_unlink, obj)
-        else:
-            print 'RELINK'
-            self.__set__(obj, value)
 
     def __on_composite_unlink(self, obj, pspec, value):
         """Unlink value if we have a part-whole (composite) relationship
@@ -411,9 +449,9 @@ class association(umlproperty):
         if pspec == '__unlink__':
             value.unlink()
             obj.connect('__unlink__', self.__on_composite_unlink, value)
-        else:
-            print 'RELINK'
-            value.relink()
+        #else:
+            #print 'RELINK'
+            #value.relink()
 
 
 class derivedunion(umlproperty):
@@ -510,28 +548,32 @@ class redefine(umlproperty):
     def __str__(self):
         return '<redefine %s: %s = %s>' % (self.name, self.type.__name__, str(self.original))
 
-    def _get(self, obj):
-        return self.original._get(obj)
-
-    def _set(self, obj, value):
-        return self.original._set(obj, value)
-
-    def _set2(self, obj, value):
-        return self.original._set2(obj, value)
-
     def __get__(self, obj, class_=None):
+        # No longer needed
         if not obj:
             return self
         return self.original.__get__(obj, class_)
 
     def __set__(self, obj, value):
+        # No longer needed
         if not isinstance(value, self.type):
             raise AttributeError, 'Value should be of type %s' % self.type.__name__
         self.original.__set__(obj, value)
 
     def __delete__(self, obj, value=None):
+        # No longer needed
         self.original.__delete__(obj, value)
 
+    def _get(self, obj):
+        return self.original._get(obj)
+
+    def _set(self, obj, value, from_opposite=False):
+        return self.original._set(obj, value, from_opposite)
+
+    def _del(self, obj, value, from_opposite=False):
+        return self.original._del(obj, value, from_opposite)
+
+
 try:
     import psyco
 except ImportError:
