@@ -12,7 +12,6 @@ del pygtk
 
 import misc.singleton
 import misc.logger
-import types
 
 import version
 
@@ -25,27 +24,29 @@ from misc.resource import Resource
 # If the resource doesn't already exist, it is created, otherwise the existing
 # resource is returned.
 resource = Resource(initial_resources = {
-			'Name': 'gaphor',
-			'Version': version.VERSION,
-			'DataDir': version.DATA_DIR,
-			'UserDataDir': version.USER_DATA_DIR
-		    })
+                        'Name': 'gaphor',
+                        'Version': version.VERSION,
+                        'DataDir': version.DATA_DIR,
+                        'UserDataDir': version.USER_DATA_DIR
+                    })
 
 class GaphorError(Exception):
     """
     Gaphor specific exception class
     """
     def __init__(self, args=None):
-	    Exception.__init__(self)
+            Exception.__init__(self)
             self.args = args
 
-def main():
+def main(gaphor_file=None):
     """Start the interactive application.
+
+    This involves importing plugins and creating the main window.
     """
     # Import stuff here, since the user might not need all the GUI stuff
     import gtk
-    # Load plugin definitions:
     import diagram
+    # Load plugin definitions:
     import pluginmanager
     from ui.mainwindow import MainWindow
 
@@ -59,8 +60,12 @@ def main():
     # Make the mainwindow accessable as a resource
     #gtk.threads_init()
     #gtk.threads_enter()
-    # Start with a clean nice new model
-    main_window.execute_action('FileNew')
+    log.debug('Loading model %s' % gaphor_file)
+    if gaphor_file:
+        main_window.set_filename(gaphor_file)
+        main_window.execute_action('FileRevert')
+    else:
+        main_window.execute_action('FileNew')
     gtk.main()
     #gtk.threads_leave()
     log.info('Bye!')
