@@ -1,8 +1,8 @@
-'''
-AssociationItem -- Graphical representation of an association.
-'''
 # vim:sw=4:et
 
+"""AssociationItem -- Graphical representation of an association.
+"""
+
 # TODO: for Association.postload(): in some cases where the association ends
 # are connected to the same Class, the head_end property is connected to the
 # tail end and visa versa.
@@ -21,12 +21,17 @@ from gaphor.diagram import initialize_item
 from diagramitem import DiagramItem
 from relationship import RelationshipItem
 
-class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
+from math import atan, pi, sin, cos
+
+class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable, diacanvas.CanvasEditable):
     """AssociationItem represents associations. 
     An AssociationItem has two AssociationEnd items. Each AssociationEnd item
     represents a Property (with Property.association == my association).
     """
     __gproperties__ = {
+        'show-direction': (gobject.TYPE_BOOLEAN, 'show direction',
+                            '',
+                            1, gobject.PARAM_READWRITE),
         'head':         (gobject.TYPE_OBJECT, 'head',
                          'AssociationEnd held by the head end of the association',
                          gobject.PARAM_READABLE),
@@ -41,20 +46,25 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
                          gobject.PARAM_READWRITE),
     }
 
+    FONT='sans bold 10'
+
     association_popup_menu = (
         'separator',
-        'Side _A', (
-            'Head_isNavigable',
-            'separator',
-            'Head_AggregationNone',
-            'Head_AggregationShared',
-            'Head_AggregationComposite'),
-        'Side _B', (
-            'Tail_isNavigable',
-            'separator',
-            'Tail_AggregationNone',
-            'Tail_AggregationShared',
-            'Tail_AggregationComposite')
+        'AssociationShowDirection',
+        'AssociationInvertDirection',
+        'separator',
+#        'Side _A', (
+#            'Head_isNavigable',
+#            'separator',
+#            'Head_AggregationNone',
+#            'Head_AggregationShared',
+#            'Head_AggregationComposite'),
+#        'Side _B', (
+#            'Tail_isNavigable',
+#            'separator',
+#            'Tail_AggregationNone',
+#            'Tail_AggregationShared',
+#            'Tail_AggregationComposite')
     )
 
     def __init__(self, id=None):
@@ -62,24 +72,41 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
 
         # AssociationEnds are really inseperable from the AssociationItem.
         # We give them the same id as the association item.
-        self._head_end = AssociationEnd()
+        self._head_end = AssociationEnd(end="head")
         self._head_end.set_child_of(self)
-        self._tail_end = AssociationEnd()
+        self._tail_end = AssociationEnd(end="tail")
         self._tail_end.set_child_of(self)
 
-    def save (self, save_func):
-        RelationshipItem.save(self, save_func)
-        if self._head_end.subject:
-            save_func('head_subject', self._head_end.subject)
-        if self._tail_end.subject:
-            save_func('tail_subject', self._tail_end.subject)
+        self._label = diacanvas.shape.Text()
+        self._label.set_font_description(pango.FontDescription(AssociationItem.FONT))
+        #self._label.set_alignment(pango.ALIGN_CENTER)
+        self._label.set_markup(False)
+        #self._label.set_max_width(100)
+        #self._label.set_max_height(100)
 
-    def load (self, name, value):
+        # Direction depends on the ends that hold the ownedEnd attributes.
+        self._show_direction = False
+        self._dir = diacanvas.shape.Path()
+        self._dir.set_line_width(2.0)
+        self._dir.line(((10, 0), (10, 10), (0, 5)))
+        self._dir.set_fill_color(diacanvas.color(0,0,0))
+        self._dir.set_fill(diacanvas.shape.FILL_SOLID)
+        self._dir.set_cyclic(True)
+
+    def save(self, save_func):
+        RelationshipItem.save(self, save_func)
+        self.save_property(save_func, 'show-direction')
+        if self._head_end.subject:
+            save_func('head-subject', self._head_end.subject)
+        if self._tail_end.subject:
+            save_func('tail-subject', self._tail_end.subject)
+
+    def load(self, name, value):
         # end_head and end_tail were used in an older Gaphor version
-        if name in ( 'head_end', 'head_subject' ):
+        if name in ( 'head_end', 'head_subject', 'head-subject' ):
             #type(self._head_end).subject.load(self._head_end, value)
             self._head_end.load('subject', value)
-        elif name in ( 'tail_end', 'tail_subject' ):
+        elif name in ( 'tail_end', 'tail_subject', 'tail-subject' ):
             #type(self._tail_end).subject.load(self._tail_end, value)
             self._tail_end.load('subject', value)
         else:
@@ -90,11 +117,15 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
         self._head_end.postload()
         self._tail_end.postload()
 
-    def do_set_property (self, pspec, value):
+    def do_set_property(self, pspec, value):
         if pspec.name == 'head-subject':
             self._head_end.subject = value
         elif pspec.name == 'tail-subject':
             self._tail_end.subject = value
+        elif pspec.name == 'show-direction':
+            self.preserve_property('show-direction')
+            self._show_direction = value
+            self.request_update()
         else:
             RelationshipItem.do_set_property(self, pspec, value)
 
@@ -107,6 +138,8 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
             return self._head_end.subject
         elif pspec.name == 'tail-subject':
             return self._tail_end.subject
+        elif pspec.name == 'show-direction':
+            return self._show_direction
         else:
             return RelationshipItem.do_get_property(self, pspec)
 
@@ -114,14 +147,96 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
 
     tail_end = property(lambda self: self._tail_end)
 
+    def get_popup_menu(self):
+        if self.subject:
+            return self.popup_menu + self.association_popup_menu
+        else:
+            return self.popup_menu
+
+    def invert_direction(self):
+        """Invert the direction of the association, this is done by
+        swapping the head and tail-ends subjects.
+        """
+        if not self.subject:
+            return
+
+        self.subject.memberEnd.moveDown(self.subject.memberEnd[0])
+
     def on_subject_notify(self, pspec, notifiers=()):
         RelationshipItem.on_subject_notify(self, pspec,
-                                           notifiers + ('ownedEnd',))
+                                           notifiers + ('name', 'ownedEnd', 'memberEnd'))
+
+        self.on_subject_notify__name(self.subject, pspec)
+
+    def on_subject_notify__name(self, subject, pspec):
+        #log.debug('Association name = %s' % (subject and subject.name))
+        if subject:
+            self._label.set_text(subject.name or '')
+        else:
+            self._label.set_text('')
+        self.request_update()
 
     def on_subject_notify__ownedEnd(self, subject, pspec):
         self.request_update()
 
-    def on_update (self, affine):
+    def on_subject_notify__memberEnd(self, subject, pspec):
+        self.request_update()
+
+    def update_label(self, p1, p2):
+        """Update the name label near the middle of the association.
+        """
+        w, h = self._label.to_pango_layout(True).get_pixel_size()
+
+        x = p1[0] > p2[0] and w + 2 or -2
+        x = (p1[0] + p2[0]) / 2.0 - x
+        y = p1[1] <= p2[1] and h or 0
+        y = (p1[1] + p2[1]) / 2.0 - y
+
+        self._label.set_pos((x, y))
+        #log.debug('label pos = (%d, %d)' % (x, y))
+        #return x, y, max(x + 10, x + w), max(y + 10, y + h)
+        return x, y, x + w, y + h
+
+    def update_dir(self, p1, p2):
+        """Create a small arrow near the middle of the association line and
+        let it point in the direction of self.subject.memberEnd[0].
+        Keep in mind that self.subject.memberEnd[0].class_ points to the class
+        *not* pointed to by the arrow.
+        """
+        x = p1[0] < p2[0] and -8 or 8
+        y = p1[1] >= p2[1] and -8 or 8
+        x = (p1[0] + p2[0]) / 2.0 + x
+        y = (p1[1] + p2[1]) / 2.0 + y
+        
+        try:
+            angle = atan((p1[1] - p2[1]) / (p1[0] - p2[0])) #/ pi * 180.0
+        except ZeroDivisionError:
+            angle = pi * 1.5
+
+        # Invert angle if member ends are inverted
+        if self.subject.memberEnd[0] is self._tail_end.subject:
+            angle += pi
+
+        if p1[0] < p2[0]:
+            angle += pi
+        elif p1[0] == p2[0] and p1[1] > p2[1]:
+            angle += pi
+        #log.debug('rotation angle is %s' % (angle/pi * 180.0))
+
+        sin_angle = sin(angle)
+        cos_angle = cos(angle)
+
+        def r(a, b):
+            return (cos_angle * a - sin_angle * b + x, \
+                    sin_angle * a + cos_angle * b + y)
+
+        # Create an arrow around (0, 0), so it can be easely rotated:
+        self._dir.line((r(-6, 0), r(6, -5), r(6, 5)))
+        self._dir.set_cyclic(True)
+
+        return x, y, x + 12, y + 10
+
+    def on_update(self, affine):
         """Update the shapes and sub-items of the association."""
         # Update line endings:
         head_subject = self._head_end.subject
@@ -129,25 +244,25 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
         if head_subject and tail_subject:
             # Update line ends using the aggregation and isNavigable values:
 
-            if head_subject.aggregation == intern('composite'):
+            if tail_subject.aggregation == intern('composite'):
                 self.set(has_head=1, head_a=20, head_b=10, head_c=6, head_d=6,
                          head_fill_color=diacanvas.color(0,0,0,255))
-            elif head_subject.aggregation == intern('shared'):
+            elif tail_subject.aggregation == intern('shared'):
                 self.set(has_head=1, head_a=20, head_b=10, head_c=6, head_d=6,
                          head_fill_color=diacanvas.color(255,255,255,255))
-            elif not head_subject.owningAssociation:
+            elif head_subject.class_:
                 # This side is navigable:
                 self.set(has_head=1, head_a=0, head_b=15, head_c=6, head_d=6)
             else:
                 self.set(has_head=0)
 
-            if tail_subject.aggregation == intern('composite'):
+            if head_subject.aggregation == intern('composite'):
                 self.set(has_tail=1, tail_a=20, tail_b=10, tail_c=6, tail_d=6,
                          tail_fill_color=diacanvas.color(0,0,0,255))
-            elif tail_subject.aggregation == intern('shared'):
+            elif head_subject.aggregation == intern('shared'):
                 self.set(has_tail=1, tail_a=20, tail_b=10, tail_c=6, tail_d=6,
                          tail_fill_color=diacanvas.color(255,255,255,255))
-            elif not tail_subject.owningAssociation:
+            elif tail_subject.class_:
                 # This side is navigable:
                 self.set(has_tail=1, tail_a=0, tail_b=15, tail_c=6, tail_d=6)
             else:
@@ -167,15 +282,61 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
         self.update_child(self._head_end, affine)
         self.update_child(self._tail_end, affine)
 
+        # update name label:
+        middle = len(handles)/2
+        self._label_bounds = self.update_label(handles[middle-1].get_pos_i(),
+                                               handles[middle].get_pos_i())
+
+        if self._show_direction and self.subject and self.subject.memberEnd:
+            b0 = self.update_dir(handles[middle-1].get_pos_i(),
+                                 handles[middle].get_pos_i())
+        else:
+            b0 = self.bounds
+
         # bounds calculation
         b1 = self.bounds
         b2 = self._head_end.get_bounds(self._head_end.affine)
         b3 = self._tail_end.get_bounds(self._tail_end.affine)
-        self.set_bounds((min(b1[0], b2[0], b3[0]), min(b1[1], b2[1], b3[1]),
-                         max(b1[2], b2[2], b3[2]), max(b1[3], b2[3], b3[3])))
+        bv = zip(self._label_bounds, b0, b1, b2, b3)
+        self.set_bounds((min(bv[0]), min(bv[1]), max(bv[2]), max(bv[3])))
                     
+    def on_shape_iter(self):
+        for s in RelationshipItem.on_shape_iter(self):
+            yield s
+        yield self._label
+        if self._show_direction:
+            yield self._dir
+
     # Gaphor Connection Protocol
 
+    def find_relationship(self, head_type, tail_type):
+        # First check if we do not already contain the right subject:
+        if self.subject:
+            end1 = self.subject.memberEnd[0]
+            end2 = self.subject.memberEnd[1]
+            if (end1.type is head_type and end2.type is tail_type) \
+               or (end2.type is head_type and end1.type is tail_type):
+                return
+                
+        # Find all associations and determine if the properties on the
+        # association ends have a type that points to the class.
+        Association = UML.Association
+        for assoc in gaphor.resource(UML.ElementFactory).itervalues():
+            if isinstance(assoc, Association):
+                #print 'assoc.memberEnd', assoc.memberEnd
+                end1 = assoc.memberEnd[0]
+                end2 = assoc.memberEnd[1]
+                if (end1.type is head_type and end2.type is tail_type) \
+                   or (end2.type is head_type and end1.type is tail_type):
+                    # check if this entry is not yet in the diagram
+                    # Return if the association is not (yet) on the canvas
+                    for item in assoc.presentation:
+                        if item.canvas is self.canvas:
+                            break
+                    else:
+                        return assoc
+        return None
+
     def allow_connect_handle(self, handle, connecting_to):
         """This method is called by a canvas item if the user tries to connect
         this object's handle. allow_connect_handle() checks if the line is
@@ -188,7 +349,7 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
             return True
         return False
 
-    def confirm_connect_handle (self, handle):
+    def confirm_connect_handle(self, handle):
         """This method is called after a connection is established. This method
         sets the internal state of the line and updates the data model.
         """
@@ -261,14 +422,14 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
                 self._head_end.subject = head_end
                 self._tail_end.subject = tail_end
 
-    def confirm_disconnect_handle (self, handle, was_connected_to):
+    def confirm_disconnect_handle(self, handle, was_connected_to):
         #log.debug('AssociationItem.confirm_disconnect_handle')
         if self.subject:
             # First delete the Property's at the ends, otherwise they will
             # be interpreted as attributes.
-            del self._head_end.subject
-            del self._tail_end.subject
-            del self.subject
+            self._head_end.set_subject(None)
+            self._tail_end.set_subject(None)
+            self.set_subject(None)
 
     # Groupable
 
@@ -282,11 +443,21 @@ class AssociationItem(RelationshipItem, diacanvas.CanvasGroupable):
     def on_groupable_iter(self):
         return iter([self._head_end, self._tail_end])
 
-    def get_popup_menu(self):
-        if self.subject:
-            return self.popup_menu + self.association_popup_menu
-        else:
-            return self.popup_menu
+    # Editable
+
+    def on_editable_get_editable_shape(self, x, y):
+        log.debug('association edit on (%d,%d)' % (x, y))
+        #p = (x, y)
+        #drp = diacanvas.geometry.distance_rectangle_point
+        #if drp(self._label_bounds, p) < 1.0:
+        return self._label
+
+    def on_editable_start_editing(self, shape):
+        pass
+
+    def on_editable_editing_done(self, shape, new_text):
+        if self.subject and self.subject.name != new_text:
+            self.subject.name = new_text
 
 
 class AssociationEnd(diacanvas.CanvasItem, diacanvas.CanvasEditable, DiagramItem):
@@ -312,9 +483,26 @@ class AssociationEnd(diacanvas.CanvasItem, diacanvas.CanvasEditable, DiagramItem
 
     FONT='sans 10'
 
-    def __init__(self, id=None):
+    head_popup_menu = (
+        'Head_isNavigable',
+        'separator',
+        'Head_AggregationNone',
+        'Head_AggregationShared',
+        'Head_AggregationComposite'
+    )
+
+    tail_popup_menu = (
+        'Tail_isNavigable',
+        'separator',
+        'Tail_AggregationNone',
+        'Tail_AggregationShared',
+        'Tail_AggregationComposite'
+    )
+
+    def __init__(self, id=None, end=None):
         self.__gobject_init__()
         DiagramItem.__init__(self, id)
+        self._end = end
         self.set_flags(diacanvas.COMPOSITE)
         
         font = pango.FontDescription(AssociationEnd.FONT)
@@ -335,6 +523,7 @@ class AssociationEnd(diacanvas.CanvasItem, diacanvas.CanvasEditable, DiagramItem
         self._mult_border.set_line_width(1.0)
 
         self._name_bounds = self._mult_bounds = (0, 0, 0, 0)
+        self._point1 = self._point2 = (0, 0)
 
     # Ensure we call the right connect functions:
     connect = DiagramItem.connect
@@ -345,14 +534,29 @@ class AssociationEnd(diacanvas.CanvasItem, diacanvas.CanvasEditable, DiagramItem
         DiagramItem.postload(self)
         #self.set_text()
 
+    def get_popup_menu(self):
+        if self.subject:
+            if self._end == 'head':
+                return self.head_popup_menu
+            else:
+                return self.tail_popup_menu
+        elif self.parent:
+            return self.parent.get_popup_menu()
+
     def set_text(self):
         """Set the text on the association end.
         """
         if self.subject:
-            n, m = self.subject.render()
-            self._name.set_text(n)
-            self._mult.set_text(m)
-            self.request_update()
+            try:
+                n, m = self.subject.render()
+            except ValueError:
+                # need more than 0 values to unpack: property was rendered as
+                # attribute while in a UNDO action for example.
+                pass
+            else:
+                self._name.set_text(n)
+                self._mult.set_text(m)
+                self.request_update()
 
     def set_navigable(self, navigable):
         """Change the AsociationEnd's navigability.
@@ -482,6 +686,9 @@ class AssociationEnd(diacanvas.CanvasItem, diacanvas.CanvasEditable, DiagramItem
                              p1[1] + mult_dy + mult_h)
         self._mult.set_pos((p1[0] + mult_dx, p1[1] + mult_dy))
 
+        self._point1 = p1
+        self._point2 = p2
+
     def on_subject_notify(self, pspec, notifiers=()):
         DiagramItem.on_subject_notify(self, pspec,
                         notifiers + ('aggregation', 'visibility',
@@ -533,7 +740,15 @@ class AssociationEnd(diacanvas.CanvasItem, diacanvas.CanvasEditable, DiagramItem
         drp = diacanvas.geometry.distance_rectangle_point
         d1 = drp(self._name_bounds, p)
         d2 = drp(self._mult_bounds, p)
-        return min(d1, d2)
+        try:
+            d3 = diacanvas.geometry.distance_point_point(self._point1, p)
+            d4, dummy = diacanvas.geometry.distance_line_point(self._point1, self._point2, p, 1.0, 0) #diacanvas.shape.CAP_ROUND)
+            if d3 < 15 and d4 < 5:
+                d3 = 0.0
+        except Exception, e:
+            log.error("Could not determine distance", e)
+            d3 = 1000.0
+        return min(d1, d2, d3)
 
     def on_shape_iter(self):
         if self.subject:
@@ -551,13 +766,22 @@ class AssociationEnd(diacanvas.CanvasItem, diacanvas.CanvasEditable, DiagramItem
 
     # Editable
 
+    def on_editable_get_editable_shape(self, x, y):
+        log.debug('association end edit on (%d,%d)' % (x, y))
+        if self.point_name(x, y) < 1.0:
+            return self._name 
+        elif self.point_mult(x,y) < 1.0:
+            return self._mult
+
     def on_editable_start_editing(self, shape):
         pass
 
     def on_editable_editing_done(self, shape, new_text):
         if shape in (self._name, self._mult):
             if self.subject and (shape == self._name or new_text != ''):
+                self.canvas.get_undo_manager().begin_transaction()
                 self.subject.parse(new_text)
+                self.canvas.get_undo_manager().commit_transaction()
             #self.set_text()
             #log.info('editing done')
 
