@@ -12,41 +12,6 @@ from gaphor.UML import Element, Presentation
 from gaphor.UML.properties import association
 
 
-class diagramassociation(association):
-    """Specialized association for use in diagram items.
-    It has the same interface as the association property defined in
-    gaphor/UML/properties.py, but delegates everything to the GObject property
-    handlers.
-    """
-
-    # TODO: Maybe we should not break our side of the association...
-    #       Since signals are still connected as the diagram item is unlinked.
-
-    def unlink(self, obj):
-        #print 'diagramassociation.unlink', obj, value
-        obj.preserve_property(self.name)
-        association.unlink(self, obj)
-
-    def _set2(self, obj, value):
-        #print 'diagramassociation._set2', obj, value
-        obj.preserve_property(self.name)
-        if obj.canvas and obj.canvas.in_undo and len(value.presentation) == 0:
-            print 'diagramassociation._set2(): relinking!'
-            value.relink()
-        return association._set2(self, obj, value)
-
-    def _del(self, obj, value):
-        #print 'diagramassociation._del', obj, value, value.id
-        obj.preserve_property(self.name)
-        # TODO: Add some extra notification here to tell the diagram
-        # item that the reference is about to be removed.
-        association._del(self, obj, value)
-        if len(value.presentation) == 0 or \
-           len(value.presentation) == 1 and obj in value.presentation:
-            #log.debug('diagramassociation._del: No more presentations: unlinking')
-            value.unlink()
-        
-
 class DiagramItem(Presentation):
     """Basic functionality for all model elements (lines and elements!).
 
@@ -75,10 +40,6 @@ class DiagramItem(Presentation):
                        gobject.TYPE_NONE, (gobject.TYPE_STRING,))
     }
 
-    # Override the original subject as defined in UML.Presentation:
-    # Note that subject calls GObject.notify to emit changes
-    subject = diagramassociation('subject', Element, upper=1, opposite='presentation')
-
     popup_menu = ()
 
     def __init__(self, id=None):
@@ -99,11 +60,7 @@ class DiagramItem(Presentation):
 
     def do_set_property(self, pspec, value):
         if pspec.name == 'subject':
-            #print 'set subject:', value
-            if value:
-                self.subject = value
-            elif self.subject:
-                del self.subject
+            self.set_subject(value)
         else:
             raise AttributeError, 'Unknown property %s' % pspec.name
 
@@ -139,17 +96,18 @@ class DiagramItem(Presentation):
         """
         #log.debug('DiagramItem.unlink(%s)' % self)
         # emit the __unlink__ signal the way UML.Element would have done:
+        self.set_subject(None)
+
         self.emit('__unlink__', '__unlink__')
-        # remove the subject if we have one
-        if self.subject:
-            del self.subject
+
+
         self.set_property('parent', None)
 
-    def relink(self):
-        """Relinking is done by popping the undo stack...
-        """
-        log.info('RELINK DiagramItem')
-        #self.emit('__unlink__', '__relink__')
+#    def relink(self):
+#        """Relinking is done by popping the undo stack...
+#        """
+#        log.info('RELINK DiagramItem')
+#        #self.emit('__unlink__', '__relink__')
 
     # gaphor.UML.Element like signal interface:
 
@@ -206,6 +164,23 @@ class DiagramItem(Presentation):
         for name in names:
             self.save_property(save_func, name)
 
+    def set_subject(self, subject=None):
+        """Set the subject. In addition, if there are no more presentations
+        on the subject, the subject is unlink()'ed.
+        """
+        #log.debug('set_subject %s %s' % (self.subject, subject))
+        old = self.subject
+
+        # remove the subject if we have one
+        if self.subject:
+            del self.subject
+
+        if old and len(old.presentation) == 0:
+            #log.debug('diagramitem.unlink: No more presentations: unlinking')
+            old.unlink()
+         
+        self.subject = subject
+
     def get_popup_menu(self):
         return self.popup_menu
 
@@ -215,10 +190,12 @@ class DiagramItem(Presentation):
 
         See: DiagramItem.on_subject_notify()
         """
+        #log.debug('_subject_connect_helper: %s %s %s' % (element, callback_prefix, prop_list))
+
         prop = prop_list[0]
         callback_name = '%s_%s' % (callback_prefix, prop)
         if len(prop_list) == 1:
-            #log.debug('_subject_connect_helper - %s' % callback_name)
+            #log.debug('_subject_connect_helper - %s %s' % (element, callback_name))
             handler = getattr(self, callback_name)
             element.connect(prop, handler)
             # Call the handler, so it can update its state
@@ -226,9 +203,9 @@ class DiagramItem(Presentation):
             #handler(element, getattr(type(element), prop))
         else:
             p = getattr(element, prop)
-            #log.debug('_subject_connect_helper 2 - %s' % prop)
+            #log.debug('_subject_connect_helper 2 - %s: %s' % (prop, p))
             pl = prop_list[1:]
-            element.connect(prop, self._on_subject_notify_helper, callback_name, pl, p)
+            element.connect(prop, self._on_subject_notify_helper, callback_name, pl, [p])
             if p:
                 self._subject_connect_helper(p, callback_name, pl)
             else:
@@ -239,10 +216,11 @@ class DiagramItem(Presentation):
 
         See: DiagramItem.on_subject_notify()
         """
+        #log.debug('_subject_disconnect_helper: %s %s %s' % (element, callback_prefix, prop_list))
         prop = prop_list[0]
         callback_name = '%s_%s' % (callback_prefix, prop)
         if len(prop_list) == 1:
-            #log.debug('_subject_disconnect_helper - %s' % callback_name)
+            #log.debug('_subject_disconnect_helper - %s %s' % (element, callback_name))
             handler = getattr(self, callback_name)
             element.disconnect(handler)
             # Call the handler, so it can update its state
@@ -251,17 +229,17 @@ class DiagramItem(Presentation):
             p = getattr(element, prop)
             #log.debug('_subject_disconnect_helper 2 - %s' % prop)
             pl = prop_list[1:]
-            element.disconnect(self._on_subject_notify_helper, callback_name, pl, p)
+            element.disconnect(self._on_subject_notify_helper, callback_name, pl, [p])
             if p:
                 self._subject_disconnect_helper(p, callback_name, pl)
             else:
-                # TODO: Maybe do an update here to, if p is None.
                 pass
 
     def _on_subject_notify_helper(self, element, pspec, callback_name, prop_list, old):
         """This signal handler handles signals that are not direct properties
-        of self.subject (e.g. 'subject.lowerValue.value'). This way the presentation class is
-        not bothered with the details of keeping track of those properties.
+        of self.subject (e.g. 'subject.lowerValue.value'). This way the
+        presentation class is not bothered with the details of keeping track
+        of those properties.
 
         NOTE: This only works for properties with multiplicity [0..1] or [1].
 
@@ -269,12 +247,16 @@ class DiagramItem(Presentation):
         """
         name = pspec.name
         prop = getattr(element, name)
-        if prop is not old:
-            # Attach a new signal handler with the new 'old' value:
-            if old:
-                self._subject_disconnect_helper(old, callback_name, prop_list)
-            if prop:
-                self._subject_connect_helper(prop, callback_name, prop_list)
+        #log.debug('_on_subject_notify_helper: %s %s %s %s %s' % (element, name, callback_name, prop_list, old))
+        # Attach a new signal handler with the new 'old' value:
+        if old[0]:
+            #log.info('disconnecting')
+            self._subject_disconnect_helper(old[0], callback_name, prop_list)
+        if prop:
+            self._subject_connect_helper(prop, callback_name, prop_list)
+
+        # Set the new "old" value
+        old[0] = prop
 
     def on_subject_notify(self, pspec, notifiers=()):
         """A new subject is set on this model element.
@@ -304,6 +286,7 @@ class DiagramItem(Presentation):
                 #log.debug('DiaCanvasItem.on_subject_notify: %s' % signal)
                 #self._subject_connect(self.subject, n)
                 subject_connect_helper(subject, callback_prefix, n)
+
         # Execute some sort of ItemNewSubject action
         try:
             main_window = resource('MainWindow')
