@@ -1,12 +1,15 @@
-# vim: sw=4:et
+# vim:sw=4:et
 """
 Commands related to the Diagram (DiaCanvas)
 """
 
+from __future__ import generators
+
 import diacanvas
 import gaphor
 import gaphor.diagram
 import gaphor.UML as UML
+from gaphor.undomanager import UndoTransactionAspect, weave_method
 from gaphor.misc.action import Action, CheckAction, RadioAction, ObjectAction
 from gaphor.misc.action import register_action
 
@@ -17,7 +20,8 @@ from dependency import DependencyItem
 from operation import OperationItem
 from nameditem import NamedItem
 from interface import InterfaceItem
-from association import AssociationEnd
+from association import AssociationItem, AssociationEnd
+
 
 class NoFocusItemError(gaphor.GaphorError):
     pass
@@ -35,6 +39,12 @@ def get_parent_focus_item(window):
             return item
     raise NoFocusItemError, 'No item has focus.'
 
+def get_pointer(view):
+    x, y = view.get_pointer()
+    x = x + view.get_hadjustment().value
+    y = y + view.get_vadjustment().value
+    return x, y
+
 class ItemNewSubjectAction(Action):
     id = 'ItemNewSubject'
 
@@ -46,14 +56,29 @@ class ItemNewSubjectAction(Action):
 
 register_action(ItemNewSubjectAction)
 
-class ItemRenameAction(Action):
-    id = 'ItemRename'
-    label = '_Rename'
-    tooltip = 'Rename selected item'
+class EditItemAction(Action):
+    id = 'EditItem'
+    label = 'Edit'
+    tooltip='Edit'
 
     def init(self, window):
         self._window = window
 
+    def execute(self):
+        # Stay backwards compatible:
+        view = self._window.get_current_diagram_view()
+        wx, wy = view.window_to_world(*get_pointer(view))
+        #log.debug('focus item %s on point (%d, %d) -> (%d, %d)' % (view.focus_item.item, x, y, wx, wy))
+        view.start_editing(view.focus_item, wx, wy)
+
+register_action(EditItemAction, 'ItemFocus')
+
+
+class RenameItemAction(EditItemAction):
+    id = 'RenameItem'
+    label = '_Rename'
+    tooltip = 'Rename selected item'
+
     def update(self):
         try:
             item = get_parent_focus_item(self._window)
@@ -63,27 +88,7 @@ class ItemRenameAction(Action):
             if isinstance(item, NamedItem):
                 self.sensitive = True
 
-    def execute(self):
-        item = self._window.get_current_diagram_view().focus_item.item
-        item.edit()
-
-register_action(ItemRenameAction, 'ItemFocus')
-
-
-class EditItemAction(Action):
-    id = 'EditItem'
-    label = 'Edit'
-    tooltip='Edit'
-
-    def init(self, window):
-        self._window = window
-
-    def execute(self):
-        item = self._window.get_current_diagram_view().focus_item.item
-        #assert isinstance(subject, (UML.Property, UML.Operation))
-        item.edit()
-
-register_action(EditItemAction, 'ItemFocus')
+register_action(RenameItemAction, 'ItemFocus')
 
 
 class AbstractClassAction(CheckAction):
@@ -107,6 +112,7 @@ class AbstractClassAction(CheckAction):
         item = get_parent_focus_item(self._window)
         item.subject.isAbstract = self.active
 
+weave_method(AbstractClassAction.execute, UndoTransactionAspect)
 register_action(AbstractClassAction, 'ItemFocus')
 
 
@@ -132,21 +138,28 @@ class CreateAttributeAction(Action):
                 self.sensitive = item.get_property('show-attributes')
 
     def execute(self):
+        view = self._window.get_current_diagram_view()
         focus_item = get_parent_focus_item(self._window)
         subject = focus_item.subject
         assert isinstance(subject, (UML.Class, UML.Interface))
         elemfact = gaphor.resource(UML.ElementFactory)
+        
         attribute = elemfact.create(UML.Property)
-        attribute.name = 'new'
+        attribute.parse('new')
         subject.ownedAttribute = attribute
+
         # Select this item for editing
         presentation = attribute.presentation
         focus_item.update_now()
+
+        wx, wy = view.window_to_world(*get_pointer(view))
         for f in focus_item.groupable_iter():
             if f in presentation:
-                f.edit()
+                vf = view.find_view_item(f)
+                view.start_editing(vf, wx, wy)
                 break
 
+weave_method(CreateAttributeAction.execute, UndoTransactionAspect)
 register_action(CreateAttributeAction, 'ShowAttributes', 'ItemFocus')
 
 
@@ -168,21 +181,27 @@ class CreateOperationAction(Action):
                 self.sensitive = item.get_property('show-operations')
 
     def execute(self):
+        view = self._window.get_current_diagram_view()
         focus_item = get_parent_focus_item(self._window)
         subject = focus_item.subject
         assert isinstance(subject, UML.Classifier)
         elemfact = gaphor.resource(UML.ElementFactory)
+
         operation = elemfact.create(UML.Operation)
-        operation.name = 'new'
+        operation.parse('new()')
         subject.ownedOperation = operation
         # Select this item for editing
         presentation = operation.presentation
         focus_item.update_now()
+
+        wx, wy = view.window_to_world(*get_pointer(view))
         for f in focus_item.groupable_iter():
             if f in presentation:
-                f.edit()
+                vf = view.find_view_item(f)
+                view.start_editing(vf, wx, wy)
                 break
 
+weave_method(CreateOperationAction.execute, UndoTransactionAspect)
 register_action(CreateOperationAction, 'ShowOperations', 'ItemFocus')
 
 
@@ -197,11 +216,13 @@ class DeleteFeatureAction(Action):
         #assert isinstance(subject, (UML.Property, UML.Operation))
         item.subject.unlink()
 
+
 class DeleteAttributeAction(DeleteFeatureAction):
     id = 'DeleteAttribute'
     label = 'Delete A_ttribute'
     tooltip='Delete the selected attribute'
 
+weave_method(DeleteAttributeAction.execute, UndoTransactionAspect)
 register_action(DeleteAttributeAction, 'ShowAttributes', 'CreateAttribute', 'ItemFocus')
 
 
@@ -210,6 +231,7 @@ class DeleteOperationAction(DeleteFeatureAction):
     label = 'Delete O_peration'
     tooltip = 'Delete the selected operation'
 
+weave_method(DeleteOperationAction.execute, UndoTransactionAspect)
 register_action(DeleteOperationAction, 'ShowOperations', 'CreateOperation', 'ItemFocus')
 
 
@@ -234,6 +256,7 @@ class ShowAttributesAction(CheckAction):
         item = get_parent_focus_item(self._window)
         item.set_property('show-attributes', self.active)
 
+weave_method(DeleteOperationAction.execute, UndoTransactionAspect)
 register_action(ShowAttributesAction, 'ItemFocus')
 
 
@@ -259,6 +282,7 @@ class ShowOperationsAction(CheckAction):
         item = get_parent_focus_item(self._window)
         item.set_property('show-operations', self.active)
 
+weave_method(ShowOperationsAction.execute, UndoTransactionAspect)
 register_action(ShowOperationsAction, 'ItemFocus')
 
 #
@@ -277,7 +301,7 @@ class SegmentAction(Action):
         assert isinstance(fi, diacanvas.CanvasLine)
         #x = view.event()
         #print 'event =', event
-        wx, wy = view.window_to_world(*view.get_pointer())
+        wx, wy = view.window_to_world(*get_pointer(view))
         x, y = fi.affine_point_w2i(wx, wy)
         segment = fi.get_closest_segment(x, y)
         return fi, segment
@@ -293,6 +317,7 @@ class AddSegmentAction(SegmentAction):
         if item:
             item.set_property('add_segment', segment)
             
+weave_method(AddSegmentAction.execute, UndoTransactionAspect)
 register_action(AddSegmentAction, 'ItemFocus')
 
 
@@ -314,6 +339,7 @@ class DeleteSegmentAction(SegmentAction):
         if item:
             item.set_property('del_segment', segment)
             
+weave_method(DeleteSegmentAction.execute, UndoTransactionAspect)
 register_action(DeleteSegmentAction, 'ItemFocus', 'AddSegment')
 
 
@@ -340,6 +366,7 @@ class OrthogonalAction(CheckAction):
             fi.set_property('add_segment', 0)
         fi.set_property('orthogonal', self.active)
 
+weave_method(OrthogonalAction.execute, UndoTransactionAspect)
 register_action(OrthogonalAction, 'ItemFocus', 'AddSegment', 'DeleteSegment')
 
 
@@ -365,6 +392,7 @@ class OrthogonalAlignmentAction(CheckAction):
         assert isinstance(fi, diacanvas.CanvasLine)
         fi.set_property('horizontal', self.active)
 
+weave_method(OrthogonalAlignmentAction.execute, UndoTransactionAspect)
 register_action(OrthogonalAlignmentAction, 'ItemFocus', 'Orthogonal')
 
 
@@ -372,6 +400,48 @@ register_action(OrthogonalAlignmentAction, 'ItemFocus', 'Orthogonal')
 # Association submenu
 #
 
+class AssociationShowDirectionAction(CheckAction):
+    id = 'AssociationShowDirection'
+    label = 'Show Direction'
+    tooltip='show direction arrow'
+
+    def init(self, window):
+        self._window = window
+
+    def update(self):
+        try:
+            item = get_parent_focus_item(self._window)
+            if isinstance(item, AssociationItem):
+                self.active = item.get_property('show-direction')
+        except NoFocusItemError:
+            pass
+
+    def execute(self):
+        fi = get_parent_focus_item(self._window)
+        assert isinstance(fi, AssociationItem)
+        fi.set_property('show-direction', self.active)
+
+weave_method(AssociationShowDirectionAction.execute, UndoTransactionAspect)
+register_action(AssociationShowDirectionAction, 'ItemFocus')
+
+
+class AssociationInvertDirectionAction(Action):
+    id = 'AssociationInvertDirection'
+    label = 'Invert Direction'
+    tooltip='Invert direction arrow'
+
+    def init(self, window):
+        self._window = window
+
+    def execute(self):
+        fi = get_parent_focus_item(self._window)
+        assert isinstance(fi, AssociationItem)
+        fi.invert_direction()
+
+weave_method(AssociationInvertDirectionAction.execute, UndoTransactionAspect)
+register_action(AssociationInvertDirectionAction, 'ItemFocus')
+
+
 class NavigableAction(CheckAction):
     end_name=''
     def init(self, window):
@@ -383,7 +453,6 @@ class NavigableAction(CheckAction):
     def update(self):
         try:
             item = get_parent_focus_item(self._window)
-            from association import AssociationItem
             if isinstance(item, AssociationItem):
                 end = item.get_property(self.end_name)
                 if end.subject:
@@ -397,6 +466,8 @@ class NavigableAction(CheckAction):
         assert isinstance(item.subject, UML.Property)
         item.set_navigable(self.active)
 
+weave_method(NavigableAction.execute, UndoTransactionAspect)
+
 
 class HeadNavigableAction(NavigableAction):
     id = 'Head_isNavigable'
@@ -422,7 +493,6 @@ class AggregationAction(RadioAction):
     def update(self):
         try:
             item = get_parent_focus_item(self._window)
-            from association import AssociationItem
             if isinstance(item, AssociationItem):
                 end = item.get_property(self.end_name)
                 if end.subject:
@@ -436,6 +506,7 @@ class AggregationAction(RadioAction):
             assert isinstance(subject, UML.Property)
             subject.aggregation = self.aggregation
 
+weave_method(AggregationAction.execute, UndoTransactionAspect)
 
 class HeadNoneAction(AggregationAction):
     id = 'Head_AggregationNone'
@@ -572,6 +643,8 @@ class DependencyTypeAction(RadioAction):
             item.set_dependency_type(self.dependency_type)
             #item.auto_dependency = False
             self._window.get_action_pool().execute('AutoDependency', active=False)
+
+weave_method(DependencyTypeAction.execute, UndoTransactionAspect)
         
 
 class DependencyTypeDependencyAction(DependencyTypeAction):
@@ -630,6 +703,7 @@ class AutoDependencyAction(CheckAction):
         item = get_parent_focus_item(self._window)
         item.auto_dependency = self.active
 
+weave_method(AutoDependencyAction.execute, UndoTransactionAspect)
 register_action(AutoDependencyAction, 'ItemFocus')
 
 
@@ -654,6 +728,7 @@ class IndirectlyInstantiatedComponentAction(CheckAction):
         item = get_parent_focus_item(self._window)
         item.subject.isIndirectlyInstantiated = self.active
 
+weave_method(IndirectlyInstantiatedComponentAction.execute, UndoTransactionAspect)
 register_action(IndirectlyInstantiatedComponentAction, 'ItemFocus')
 
 
@@ -665,11 +740,9 @@ class MoveAction(Action):
         return self._window.get_current_diagram_view() \
             .focus_item.item
 
-
     def _getParent(self):
         return get_parent_focus_item(self._window)
 
-
     def _getElements(self, cls, item):
         if isinstance(item, AttributeItem):
             collection = cls.ownedAttribute
@@ -677,11 +750,9 @@ class MoveAction(Action):
             collection = cls.ownedOperation
         return collection
 
-
     def init(self, window):
         self._window = window
 
-
     def update(self):
         try:
             cls_item = self._getParent()
@@ -693,7 +764,6 @@ class MoveAction(Action):
                 self.active = item.subject 
                 self.sensitive = self._isSensitive(cls_item.subject, item)
 
-
     def execute(self):
         cls = self._getParent().subject
         item = self._getItem()
@@ -706,6 +776,7 @@ class MoveAction(Action):
         move(item.subject)
         self._window.execute_action('ItemFocus')
 
+weave_method(MoveAction.execute, UndoTransactionAspect)
 
 class MoveUpAction(MoveAction):
     id = 'MoveUp'
@@ -733,11 +804,10 @@ class MoveDownAction(MoveAction):
 register_action(MoveDownAction, 'ItemFocus')
 
 
-class DummyItem(UML.Element, UML.Presentation):
-    pass
-
-class Fold(Action):
-    accel = 'C-f'
+class FoldAction(Action):
+    id = 'Fold'
+    label = '_Fold'
+    tooltip = 'Hide details'
 
     def init(self, window):
         self._window = window
@@ -748,101 +818,36 @@ class Fold(Action):
         except NoFocusItemError:
             pass
         else:
-            self.sensitive = self.isSensitive(item)
-
+            self.sensitive = isinstance(item, InterfaceItem)
 
     def execute(self):
         item = get_parent_focus_item(self._window)
-        log.debug('Action %s: %s' % (self.id, item.subject.name))
+        #log.debug('Action %s: %s' % (self.id, item.subject.name))
 
-        # create interface diagram element and assign model element to
-        # diagram element
-        diag = self._window.get_current_diagram()
-        new_item = self.newElement(diag)
-        new_item.subject = item.subject
+        item.set_property('drawing-style', InterfaceItem.DRAW_ICON)
 
-        # center new element diagram in the centre of old one
-        x, y = item.get_property('affine')[4:]
-        new_item.update_now()
-        new_item.move(x + (item.width - new_item.width) / 2.0,
-                    y + (item.height - new_item.height) / 2.0)
-
-        # Create a dummy presentation, since we should keep tract of the items subject
-        dummy = DummyItem()
-        # Some extra dummy presentations for association ends
-        dummy_head_end = DummyItem()
-        dummy_tail_end = DummyItem()
-
-        # Find all elements that are connected to our item
-        # (Should become (added 10-6-2004 to diacanvas2))
-        #for connected_handle in item.connected_handles:
-        for connected_item in item.canvas.select(lambda i: i.handles and \
-                                                 (i.handles[0].connected_to is item or \
-                                                  i.handles[-1].connected_to is item)):
-            #connected_item = connected_handle.owner
-
-            #print 'connected item', connected_item
-            # Store the subject, in case of an association also store the
-            # head and tail ends subjects
-            if connected_item.subject:
-                dummy.subject = connected_item.subject
-                if isinstance(dummy.subject, UML.Association):
-                    dummy_head_end.subject = connected_item.get_property('head_subject')
-                    dummy_tail_end.subject = connected_item.get_property('tail_subject')
-
-            # This is the main part. First disconnect, then restore the subject (like if we are
-            # loading the model) and at last connect to the new item.
-            if connected_item.handles[0].connected_to is item:
-                item.disconnect_handle(connected_item.handles[0])
-                connected_item.subject = dummy.subject
-                new_item.connect_handle(connected_item.handles[0])
-            if connected_item.handles[-1].connected_to is item:
-                item.disconnect_handle(connected_item.handles[-1])
-                connected_item.subject = dummy.subject
-                new_item.connect_handle(connected_item.handles[-1])
-
-            assert connected_item.subject is dummy.subject
-
-            # Remove our connected dummy items
-            if isinstance(dummy.subject, UML.Association):
-                connected_item.set_property('head-subject', dummy_head_end.subject)
-                connected_item.set_property('tail-subject', dummy_tail_end.subject)
-                del dummy_head_end.subject, dummy_tail_end.subject
-            del dummy.subject
-
-        # remove old diagram element
-        item.unlink()
+weave_method(FoldAction.execute, UndoTransactionAspect)
+register_action(FoldAction, 'ItemFocus')
 
 
-class UnfoldAction(Fold):
+class UnfoldAction(FoldAction):
     id = 'Unfold'
     label = '_Unfold'
     tooltip = 'View details'
 
-    def isSensitive(self, item):
-        return isinstance(item, InterfaceItem)
+    def execute(self):
+        item = get_parent_focus_item(self._window)
+        #log.debug('Action %s: %s' % (self.id, item.subject.name))
 
-    def newElement(self, diag):
-        return diag.create(gaphor.diagram.ClassItem)
+        item.set_property('drawing-style', InterfaceItem.DRAW_COMPARTMENT)
+        # Make sure lines are updated properly:
+        item.canvas.update_now()
+        item.canvas.update_now()
 
+weave_method(UnfoldAction.execute, UndoTransactionAspect)
 register_action(UnfoldAction, 'ItemFocus')
 
 
-class FoldAction(Fold):
-    id = 'Fold'
-    label = '_Fold'
-    tooltip = 'Hide details'
-
-    def isSensitive(self, item):
-        return isinstance(item, ClassItem) and isinstance(item.subject, UML.Interface)
-
-    def newElement(self, diag):
-        return diag.create(gaphor.diagram.InterfaceItem)
-
-
-register_action(FoldAction, 'ItemFocus')
-
-
 class ApplyStereotypeAction(CheckAction, ObjectAction):
 
     def __init__(self, stereotype):
@@ -861,8 +866,10 @@ class ApplyStereotypeAction(CheckAction, ObjectAction):
             pass
         else:
             self.sensitive = isinstance(item, ClassItem)
-            if self.sensitive:
+            if self.sensitive and item.subject:
                 self.active = self.stereotype in item.subject.appliedStereotype
+            else:
+                self.active = False
 
     def execute(self):
         item = get_parent_focus_item(self._window)
@@ -871,3 +878,93 @@ class ApplyStereotypeAction(CheckAction, ObjectAction):
         else:
             del item.subject.appliedStereotype[self.stereotype]
 
+weave_method(ApplyStereotypeAction.execute, UndoTransactionAspect)
+
+
+class CreateLinksAction(Action):
+    """Create links (associations, generalizations, dependencies) from
+    the selected diagram items to other items on the same diagram. Those
+    links have to exist in the data layer of course.
+    """
+    id = 'CreateLinks'
+    label = '_Create Links'
+    tooltip = 'Make existing relationships between diagram items visible'
+
+    def init(self, window):
+        self._window = window
+
+    def update(self):
+        diagram_tab = self._window.get_current_diagram_tab()
+        self.sensitive = diagram_tab and len(diagram_tab.get_view().selected_items) > 0
+
+    def connect_relationship(self, rel, head_item, tail_item):
+        """Connect the lines, the the Item's connect handler figure out
+        which model element should be used as subject.
+        """
+        def find_center(item):
+            """Find the center point of the item, in world coordinates
+            """
+            x = item.width / 2.0
+            y = item.height / 2.0
+            return item.affine_point_i2w(x, y)
+
+        center0 = find_center(head_item)
+        center1 = find_center(tail_item)
+        center = (center0[0] + center1[0]) / 2.0, (center0[1] + center1[1]) / 2.0
+        rel.handles[0].set_pos_w(*center)
+        rel.handles[-1].set_pos_w(*center)
+
+        head_item.connect_handle(rel.handles[0])
+        tail_item.connect_handle(rel.handles[-1])
+
+    def create_missing_relationships(self, item, diagram, item_type):
+        new_rel = diagram.create(item_type)
+        for other_item in diagram.canvas.root.children:
+            if not other_item.subject:
+                continue
+
+            try:
+                while new_rel.find_relationship(item.subject, other_item.subject):
+                    self.connect_relationship(new_rel, item, other_item)
+                    #item.connect_handle(new_rel.handles[0])
+                    #other_item.connect_handle(new_rel.handles[-1])
+                    # Create a new item we want to connect
+                    new_rel = diagram.create(item_type)
+            except AttributeError:
+                pass
+
+            # Try the other way too:
+            try:
+                while new_rel.find_relationship(other_item.subject, item.subject):
+                    self.connect_relationship(new_rel, other_item, item)
+                    #other_item.connect_handle(new_rel.handles[0])
+                    #item.connect_handle(new_rel.handles[-1])
+                    # Create a new item we want to connect
+                    new_rel = diagram.create(item_type)
+            except AttributeError:
+                pass
+
+        # We always create one relationship to much. Remove it.
+        new_rel.unlink()
+
+    def execute(self):
+        diagram_tab = self._window.get_current_diagram_tab()
+        diagram = diagram_tab.get_diagram()
+        for view_item in diagram_tab.get_view().selected_items:
+            item = view_item.item
+            if isinstance(item, gaphor.diagram.ClassItem):
+                self.create_missing_relationships(item, diagram,
+                                            gaphor.diagram.AssociationItem)
+
+            if isinstance(item, gaphor.diagram.ClassifierItem):
+                self.create_missing_relationships(item, diagram,
+                                            gaphor.diagram.ImplementationItem)
+                self.create_missing_relationships(item, diagram,
+                                            gaphor.diagram.GeneralizationItem)
+
+            self.create_missing_relationships(item, diagram,
+                                              gaphor.diagram.DependencyItem)
+
+weave_method(CreateLinksAction.execute, UndoTransactionAspect)
+register_action(CreateLinksAction, 'ItemFocus', 'ItemSelect')
+
