@@ -8,46 +8,20 @@ import gobject
 import gtk
 import gaphor.UML as UML
 import gaphor.diagram as diagram
+import gaphor.undomanager as undomanager
 import gc
 import traceback
 from threading import Thread
-import Queue
 
 import gaphor
 from gaphor.misc.action import Action, CheckAction, RadioAction, register_action
+from gaphor.misc.gidlethread import GIdleThread, Queue, QueueEmpty
+from gaphor.misc.xmlwriter import XMLWriter
+from gaphor.misc.errorhandler import error_handler, ErrorHandlerAspect, weave_method
 from gaphor.i18n import _
 
 DEFAULT_EXT='.gaphor'
 
-class WorkerThread(Thread):
-    """Run a function within a thread. The worker thread has a blocking
-    start_blocking() function, which calls the GObject main loop every
-    now and then.
-    """
-
-    def __init__(self, function):
-        Thread.__init__(self)
-        #self.setDaemon(True)
-        self.function = function
-        self.error = None
-
-    def start_blocking(self):
-        """The thread is launched, however, as long as the thread is active
-        the main loop is ran.
-        """
-        Thread.start(self)
-        main = gobject.main_context_default()
-        # Run main iterations as long as this thread exists.
-        while self.isAlive():
-            main.iteration(False)
-
-    def run(self):
-        try:
-            self.function()
-        except Exception, e:
-            log.error(_('Error occured while in worker thread'), e)
-            self.error = e
-
 def show_status_window(title, message, parent=None, queue=None):
     win = gtk.Window(gtk.WINDOW_TOPLEVEL)
     win.set_title(title)
@@ -56,8 +30,8 @@ def show_status_window(title, message, parent=None, queue=None):
     win.set_modal(True)
     win.set_resizable(False)
     win.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
-    win.set_skip_taskbar_hint(True)
-    win.set_skip_pager_hint(True)
+    #win.set_skip_taskbar_hint(True)
+    #win.set_skip_pager_hint(True)
     win.set_border_width(24)
     vbox = gtk.VBox(spacing=24)
     win.add(vbox)
@@ -72,15 +46,16 @@ def show_status_window(title, message, parent=None, queue=None):
         percentage = 0
         try:
             while True:
-                percentage = queue.get_nowait()
-        except Queue.Empty:
+                percentage = queue.get()
+        except QueueEmpty:
             pass
         if percentage:
             progress_bar.set_fraction(percentage / 100.0)
         return True
 
     if queue:
-        idle_id = gobject.idle_add(progress_idle_handler, progress_bar, queue, priority=gobject.PRIORITY_LOW)
+        idle_id = gobject.idle_add(progress_idle_handler, progress_bar, queue,
+                                   priority=gobject.PRIORITY_LOW)
         # Make sure the idle fucntion is removed as soon as the window
         # is destroyed.
         def remove_progress_idle_handler(window, idle_id):
@@ -101,6 +76,17 @@ class NewAction(Action):
         self._window = window
 
     def execute(self):
+        factory = gaphor.resource(UML.ElementFactory)
+        if factory.size():
+            dialog = gtk.MessageDialog(self._window.get_window(),
+                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
+                _("Opening a new model will flush the currently loaded model.\nAny changes made will not be saved. Do you want to continue?"))
+            answer = dialog.run()
+            dialog.destroy()
+            if answer != gtk.RESPONSE_YES:
+                return
+
         factory = gaphor.resource(UML.ElementFactory)
         factory.flush()
         gc.collect()
@@ -115,65 +101,97 @@ class NewAction(Action):
         self._window.set_message(_('Created a new model'))
         factory.notify_model()
 
-        path = self._window.get_model().path_from_element(diagram)
-        # Expand the first row:
-        self._window.get_tree_view().expand_row(path[:-1], False)
-        # Select the diagram, so it can be opened by the OpenModelElement action
-        selection = self._window.get_tree_view().get_selection()
-        selection.select_path(path)
+        self._window.select_element(diagram)
         self._window.execute_action('OpenModelElement')
 
+weave_method(NewAction.execute, ErrorHandlerAspect, message='Could not create a new model.')
 register_action(NewAction)
 
-
-class OpenAction(Action):
-
-    id = 'FileOpen'
-    label = '_Open...'
-    stock_id='gtk-open'
-    tooltip = 'Load a Gaphor project from a file'
+class RevertAction(Action):
+    id = 'FileRevert'
+    label = '_Revert...'
+    stock_id='gtk-revert'
+    tooltip = 'Reload the loaded Gaphor project from file'
 
     def init(self, window):
         self.filename = None
         self._window = window
 
     def execute(self):
-        filesel = gtk.FileSelection('Open Gaphor file')
-        filesel.hide_fileop_buttons()
+        filename = self._window.get_filename()
+
+        action_states = self._window.action_pool.get_action_states()
+        try:
+            import gaphor.storage as storage
+            log.debug('Loading from: %s' % filename)
+            queue = Queue()
+            win = show_status_window(_('Loading...'), _('Loading model from %s') % filename, self._window.get_window(), queue)
+            self.filename = filename
+            gc.collect()
+            worker = GIdleThread(storage.load_generator(filename), queue)
+            self._window.action_pool.insensivate_actions()
+            undomanager.get_undo_manager().clear_undo_stack()
+            undomanager.get_undo_manager().clear_redo_stack()
+            worker.start()
+            worker.wait()
+            if worker.error:
+                log.error('Error while loading model from file %s: %s' % (filename, worker.error))
+                error_handler(message='Error while loading model from file %s' % filename, exc_info=worker.exc_info)
+
+            self._window.set_message('Model loaded successfully')
+            model = self._window.get_model()
+            view = self._window.get_tree_view()
+
+            # Expand all root elements:
+            for node in model.root[1]:
+                view.expand_row(model.path_from_element(node[0]), False)
+
+        finally:
+            self._window.action_pool.set_action_states(action_states)
+            self._window.action_pool.update_actions()
+            try:
+                win.destroy()
+            except:
+                pass
+
+weave_method(RevertAction.execute, ErrorHandlerAspect, message='Could not load model file.')
+register_action(RevertAction)
+
+
+class OpenAction(RevertAction):
+
+    id = 'FileOpen'
+    label = '_Open...'
+    stock_id='gtk-open'
+    tooltip = 'Load a Gaphor project from a file'
+
+    def execute(self):
+        if gtk.gtk_version < (2, 4, 0):
+            filesel = gtk.FileSelection('Open Gaphor model')
+            filesel.hide_fileop_buttons()
+        else:
+            filesel = gtk.FileChooserDialog(title='Open Gaphor model',
+                                            action=gtk.FILE_CHOOSER_ACTION_OPEN,
+                                            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
+
+            filter = gtk.FileFilter()
+            filter.set_name("Gaphor models")
+            filter.add_pattern("*.gaphor")
+            filesel.add_filter(filter)
+
+            filter = gtk.FileFilter()
+            filter.set_name("All files")
+            filter.add_pattern("*")
+            filesel.add_filter(filter)
+
         filesel.set_filename(self.filename or '')
 
         response = filesel.run()
         filename = filesel.get_filename()
         filesel.destroy()
         if filename and response == gtk.RESPONSE_OK:
-            action_states = self._window.action_pool.get_action_states()
-            try:
-                import gaphor.storage as storage
-                log.debug('Loading from: %s' % filename)
-                queue = Queue.Queue()
-                win = show_status_window(_('Loading...'), _('Loading model from %s') % filename, self._window.get_window(), queue)
-                self.filename = filename
-                gc.collect()
-                worker = WorkerThread(lambda: storage.load(filename, status_queue=lambda m: queue.put_nowait(m) and 1))
-                self._window.action_pool.insensivate_actions()
-                worker.start_blocking()
-                if worker.error:
-                    log.error('Error while loading model from file %s: %s' % (filename, worker.error))
-
-                self._window.set_filename(filename)
-                self._window.set_message('Model loaded successfully')
-                model = self._window.get_model()
-                view = self._window.get_tree_view()
-
-                # Expand all root elements:
-                for node in model.root[1]:
-                    view.expand_row(model.path_from_element(node[0]), False)
-
-                # Restore states of actions
-                win.destroy()
-            finally:
-                self._window.action_pool.set_action_states(action_states)
-                self._window.action_pool.update_actions()
+            self._window.set_filename(filename)
+            RevertAction.execute(self)
 
 register_action(OpenAction)
 
@@ -185,10 +203,10 @@ class SaveAsAction(Action):
     def init(self, window):
         self._window = window
         self.factory = gaphor.resource('ElementFactory')
-        self.factory.connect(self.on_element_factory)
-        self.on_element_factory(self)
+        #self.factory.connect(self.on_element_factory)
+        #self.on_element_factory(self)
         # Disconnect when the window is closed:
-        window.connect(self.on_window_closed)
+        #window.connect(self.on_window_closed)
 
     def on_element_factory(self, *args):
         #factory = gaphor.resource('ElementFactory')
@@ -207,25 +225,37 @@ class SaveAsAction(Action):
             if not filename.endswith(DEFAULT_EXT):
                 filename = filename + DEFAULT_EXT
 
-            queue = Queue.Queue()
+            queue = Queue()
             log.debug('Saving to: %s' % filename)
             win = show_status_window('Saving...', 'Saving model to %s' % filename, self._window.get_window(), queue)
-            worker = WorkerThread(lambda: storage.save(filename, status_queue=lambda m: queue.put_nowait(m) and 1))
-            action_states = self._window.action_pool.get_action_states()
-            self._window.action_pool.insensivate_actions()
-            worker.start_blocking()
-            if worker.error:
-                log.error('Error while saving model to file %s: %s' % (filename, worker.error))
+            try:
+                out = open(filename, 'w')
 
-            self._window.set_filename(filename)
+                worker = GIdleThread(storage.save_generator(XMLWriter(out)), queue)
+                action_states = self._window.action_pool.get_action_states()
+                self._window.action_pool.insensivate_actions()
+                worker.start()
+                worker.wait()
+                if worker.error:
+                    log.error('Error while saving model to file %s: %s' % (filename, worker.error))
+                    error_handler(message='Error while saving model to file %s' % filename, exc_info=worker.exc_info)
+                out.close()
 
-            # Restore states of actions
-            self._window.action_pool.set_action_states(action_states)
-            win.destroy()
+                self._window.set_filename(filename)
+
+                # Restore states of actions
+                self._window.action_pool.set_action_states(action_states)
+            finally:
+                win.destroy()
 
     def execute(self):
         filename = self._window.get_filename()
-        filesel = gtk.FileSelection('Save file as')
+        if gtk.gtk_version < (2, 4, 0):
+            filesel = gtk.FileSelection('Save Gaphor model as')
+        else:
+            filesel = gtk.FileChooserDialog(title='Save Gaphor model as',
+                                            action=gtk.FILE_CHOOSER_ACTION_SAVE,
+                                            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_SAVE,gtk.RESPONSE_OK))
         filesel.set_filename(filename or '')
         response = filesel.run()
         filename = None
@@ -234,6 +264,7 @@ class SaveAsAction(Action):
         filesel.destroy()
         self.save(filename)
 
+weave_method(SaveAsAction.save, ErrorHandlerAspect, message='Could not save model to file.')
 register_action(SaveAsAction)
 
 
@@ -319,6 +350,25 @@ class OpenConsoleWindowAction(Action):
 
 register_action(OpenConsoleWindowAction)
 
+class ManualAction(Action):
+    id = 'Manual'
+    label = '_Manual'
+    stock_id = 'gtk-about'
+    tooltip='Manual for Gaphor'
+
+    def init(self, window):
+        self._window = window
+
+    def execute(self):
+        dialog = gtk.MessageDialog(self._window.get_window(),
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+            gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
+            "For the manual, have a look at:\nhttp://gaphor.sourceforge.net/manual")
+        dialog.run()
+        dialog.destroy()
+
+
+register_action(ManualAction)
 
 class AboutAction(Action):
     id = 'About'
@@ -404,12 +454,7 @@ class CreateDiagramAction(Action):
         diagram = gaphor.resource('ElementFactory').create(UML.Diagram)
         diagram.package = element
 
-        path = self._window.get_model().path_from_element(diagram)
-        # Expand the row:
-        self._window.get_tree_view().expand_row(path[:-1], False)
-        # Select the diagram, so it can be opened by the OpenModelElement action
-        selection = self._window.get_tree_view().get_selection()
-        selection.select_path(path)
+        self._window.select_element(diagram)
         self._window.execute_action('OpenModelElement')
 
 register_action(CreateDiagramAction, 'SelectRow')
@@ -540,3 +585,59 @@ class TabChangeAction(Action):
         pass
 
 register_action(TabChangeAction)
+
+class UndoStackAction(Action):
+    """Dummy action that triggers the undo and redo actions to update
+    themselves.
+    """
+    id = 'UndoStack'
+    
+    def init(self, window):
+        pass
+
+register_action(UndoStackAction)
+
+class UndoAction(Action):
+    id = 'Undo'
+    stock_id = 'gtk-undo'
+    label = '_Undo'
+    tooltip = 'Undo the most recent changes'
+    accel = 'C-z'
+
+    # TODO: check if the diagram can undo.
+
+    def init(self, window):
+        self._window = window
+
+    def update(self):
+        self.sensitive = undomanager.get_undo_manager().can_undo()
+
+    def execute(self):
+        undomanager.get_undo_manager().undo_transaction()
+        #self.update()
+        self._window.execute_action('UndoStack')
+
+register_action(UndoAction, 'UndoStack')
+
+
+class RedoAction(Action):
+    id = 'Redo'
+    stock_id = 'gtk-redo'
+    tooltip = 'Redo the undone changes'
+    accel = 'C-r'
+
+    def init(self, window):
+        self._window = window
+
+    def update(self):
+        self.sensitive = undomanager.get_undo_manager().can_redo()
+
+    def execute(self):
+        print 'RedoAction'
+        undomanager.get_undo_manager().redo_transaction()
+        #self.update()
+        self._window.execute_action('UndoStack')
+
+register_action(RedoAction, 'UndoStack')
+
+
