@@ -22,10 +22,11 @@ def _mod_and_keyval_from_accel(accel):
     keyval = 0
     modifier = 0
     if accel:
-        gtk_accel = accel.upper().replace('C-', '<Control>').replace('S-', '<Shift>').replace('M-', '<Alt>')
+        gtk_accel = accel.replace('C-', '<Control>').replace('S-', '<Shift>').replace('M-', '<Alt>')
         if gtk_accel[-1] in keyval_map.keys():
             gtk_accel = gtk_accel[:-1] + keyval_map[gtk_accel[-1]]
         keyval, modifier = gtk.accelerator_parse(gtk_accel)
+        #log.debug('Translated %s to %d' % (gtk_accel, keyval))
 
 #        accel = accel.upper()
 #        if accel.find('S-') != -1:
@@ -34,7 +35,7 @@ def _mod_and_keyval_from_accel(accel):
 #            modifier |= gtk.gdk.CONTROL_MASK
 #        if accel.find('M-') != -1:
 #            modifier |= gtk.gdk.MOD1_MASK
-#	keyval = ord(accel[-1])
+#        keyval = ord(accel[-1])
     return modifier, keyval
 
 
@@ -423,3 +424,17 @@ class MenuFactory(object):
             icon = None
         return icon, label
 
+
+def toolbox_to_menu(toolbox):
+    """Given a toolbox definition, create a list suitable for
+    inclusion in a menu.
+    """
+    l = []
+    for box in toolbox:
+        if box[0]:
+            l.append(box[0])
+            l.append(box[1])
+        else:
+            for item in box[1]:
+                l.append(item)
+    return l
