@@ -10,6 +10,7 @@ import stock
 
 import gaphor.UML as UML
 from gaphor import resource
+from gaphor.undomanager import get_undo_manager
 import operator
 
 # The following items will not be shown in the treeview, although they
@@ -47,8 +48,8 @@ class NamespaceModel(gtk.GenericTreeModel):
             return
         node = (element, [])
         parent_node[1].append(node)
-	# Sort the nodes, so we can tell the views exactly where the element
-	# is inserted.
+        # Sort the nodes, so we can tell the views exactly where the element
+        # is inserted.
         self.sort_node(parent_node)
         path = self.path_from_element(element)
         #print 'new_node_from_element', path, element, element.name
@@ -63,7 +64,7 @@ class NamespaceModel(gtk.GenericTreeModel):
 
     def detach_notifiers_from_node(self, node):
         """Detach notifiers for node
-	"""
+        """
         node[0].disconnect(self.on_name_changed)
         if isinstance(node[0], UML.Namespace):
             node[0].disconnect(self.on_ownedmember_changed)
@@ -72,7 +73,7 @@ class NamespaceModel(gtk.GenericTreeModel):
 
     def node_and_path_from_element(self, element):
         """Return (node, path) of an element.
-	"""
+        """
         #assert isinstance(element, UML.NamedElement)
         if element:
             parent_node, parent_path = self.node_and_path_from_element(element.namespace)
@@ -90,17 +91,17 @@ class NamespaceModel(gtk.GenericTreeModel):
 
     def node_from_element(self, element):
         """Get the node for an element.
-	"""
+        """
         return self.node_and_path_from_element(element)[0]
 
     def path_from_element(self, element):
         """Get the path to an element as a tuple (e.g. (0, 1, 1)).
-	"""
+        """
         return self.node_and_path_from_element(element)[1]
 
     def node_from_path(self, path):
         """Get the node form a path. None is returned if no node is found.
-	"""
+        """
         try:
             node = self.root
             for index in path:
@@ -165,7 +166,7 @@ class NamespaceModel(gtk.GenericTreeModel):
 
     def on_name_changed(self, element, pspec):
         """the name of element has changed, update the tree model
-	"""
+        """
         path = self.path_from_element(element)
         if path:
             self.row_changed(path, self.get_iter(path))
@@ -183,7 +184,6 @@ class NamespaceModel(gtk.GenericTreeModel):
         """update the tree model when the ownedMember list changes.
         Element is the object whose ownedMember property has changed.
         """
-        #print 'on_ownedmember_changed', element
         node = self.node_from_element(element)
         # The elements currently known by the tree model
         node_members = map(lambda e: e[0], node[1])
@@ -258,22 +258,22 @@ class NamespaceModel(gtk.GenericTreeModel):
 
     def on_get_flags(self):
         """Returns the GtkTreeModelFlags for this particular type of model.
-	"""
+        """
         return 0
 
     def on_get_n_columns(self):
         """Returns the number of columns in the model.
-	"""
+        """
         return 1
 
     def on_get_column_type(self, index):
         """Returns the type of a column in the model.
-	"""
+        """
         return gobject.TYPE_PYOBJECT
 
     def on_get_path (self, node):
         """Returns the path for a node as a tuple (0, 1, 1).
-	"""
+        """
         #print 'on_get_path', node
         return self.path_from_element(node[0])
 
@@ -281,13 +281,13 @@ class NamespaceModel(gtk.GenericTreeModel):
         """Returns the node corresponding to the given path.
         The path is a tuple of values, like (0 1 1). Returns None if no
         iterator can be created.
-	"""
+        """
         #print 'on_get_iter', path
         return self.node_from_path(path)
 
     def on_get_value(self, node, column):
         """Returns the model element that matches 'node'.
-	"""
+        """
         assert column == 0, 'column can only be 0'
         #print 'on_get_value', node, column
         return node[0]
@@ -295,7 +295,7 @@ class NamespaceModel(gtk.GenericTreeModel):
     def on_iter_next(self, node):
         """Returns the next node at this level of the tree (None if no
         next element).
-	"""
+        """
         #print 'on_iter_next:', node
         try:
             parent = self.node_from_element(node[0].namespace)
@@ -310,25 +310,25 @@ class NamespaceModel(gtk.GenericTreeModel):
         
     def on_iter_has_child(self, node):
         """Returns true if this node has children, or None.
-	"""
+        """
         #print 'on_iter_has_child', node
         return len(node[1]) > 0
 
     def on_iter_children(self, node):
         """Returns the first child of this node, or None.
-	"""
+        """
         #print 'on_iter_children'
         return node[1][0]
 
     def on_iter_n_children(self, node):
         """Returns the number of children of this node.
-	"""
+        """
         #print 'on_iter_n_children'
         return len (node[1])
 
     def on_iter_nth_child(self, node, n):
         """Returns the nth child of this node.
-	"""
+        """
         #print "on_iter_nth_child", node, n
         try:
             if node is None:
@@ -346,6 +346,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         return self.node_from_element(node[0].namespace)
 
 class NamespaceView(gtk.TreeView):
+
     TARGET_STRING = 0
     TARGET_ELEMENT_ID = 1
     DND_TARGETS = [
@@ -392,16 +393,16 @@ class NamespaceView(gtk.TreeView):
         self.drag_source_set(gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
                              NamespaceView.DND_TARGETS,
                              gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK)
-        self.connect('drag-data-get', NamespaceView.do_drag_data_get)
+        self.connect('drag-data-get', NamespaceView.on_drag_data_get)
 
         # drop
         #self.drag_dest_set (gtk.DEST_DEFAULT_ALL, [NamespaceView.DND_TARGETS[-1]],
         #                    gtk.gdk.ACTION_DEFAULT)
         self.enable_model_drag_dest([NamespaceView.DND_TARGETS[-1]],
                                     gtk.gdk.ACTION_DEFAULT)
-        self.connect('drag-data-received', NamespaceView.do_drag_data_received)
-        self.connect('drag-drop', NamespaceView.do_drag_drop)
-        self.connect('drag-data-delete', NamespaceView.do_drag_data_delete)
+        self.connect('drag-data-received', NamespaceView.on_drag_data_received)
+        self.connect('drag-drop', NamespaceView.on_drag_drop)
+        self.connect('drag-data-delete', NamespaceView.on_drag_data_delete)
 
     def get_selected_element(self):
         selection = self.get_selection()
@@ -445,11 +446,11 @@ class NamespaceView(gtk.TreeView):
 #    def do_drag_begin (self, context):
 #        print 'do_drag_begin'
 
-    def do_drag_data_get(self, context, selection_data, info, time):
-        """Get the data to be dropped by do_drag_data_received().
+    def on_drag_data_get(self, context, selection_data, info, time):
+        """Get the data to be dropped by on_drag_data_received().
         We send the id of the dragged element.
         """
-        #log.debug('do_drag_data_get')
+        #log.debug('on_drag_data_get')
         selection = self.get_selection()
         model, iter = selection.get_selected()
         if iter:
@@ -459,14 +460,14 @@ class NamespaceView(gtk.TreeView):
             else:
                 selection_data.set(selection_data.target, 8, element.name)
 
-    def do_drag_data_delete (self, context):
+    def on_drag_data_delete (self, context):
         """DnD magic. do not touch
         """
         self.emit_stop_by_name('drag-data-delete')
 
     # Drop
-    def do_drag_data_received(self, context, x, y, selection, info, time):
-        """Drop the data send by do_drag_data_get().
+    def on_drag_data_received(self, context, x, y, selection, info, time):
+        """Drop the data send by on_drag_data_get().
         """
         self.emit_stop_by_name('drag-data-received')
         #print 'drag_data_received'
@@ -497,7 +498,9 @@ class NamespaceView(gtk.TreeView):
 
                 # Set package. This only works for classifiers, packages and
                 # diagrams. Properties and operations should not be moved.
+                get_undo_manager().begin_transaction()
                 element.package = dest_element
+                get_undo_manager().commit_transaction()
 
             except AttributeError:
                 #print dir(context)
@@ -510,7 +513,7 @@ class NamespaceView(gtk.TreeView):
                 selection = self.get_selection()
                 selection.select_path(path)
 
-    def do_drag_drop(self, context, x, y, time):
+    def on_drag_drop(self, context, x, y, time):
         """DnD magic. do not touch
         """
         self.emit_stop_by_name('drag-drop')
